package x86_test

import (
	"testing"

	"git.urbach.dev/cli/q/src/cpu"
	"git.urbach.dev/cli/q/src/x86"
	"git.urbach.dev/go/assert"
)

func TestCallRegister(t *testing.T) {
	usagePatterns := []struct {
		Register cpu.Register
		Code     []byte
	}{
		{x86.RAX, []byte{0xFF, 0xD0}},
		{x86.RCX, []byte{0xFF, 0xD1}},
		{x86.RDX, []byte{0xFF, 0xD2}},
		{x86.RBX, []byte{0xFF, 0xD3}},
		{x86.RSP, []byte{0xFF, 0xD4}},
		{x86.RBP, []byte{0xFF, 0xD5}},
		{x86.RSI, []byte{0xFF, 0xD6}},
		{x86.RDI, []byte{0xFF, 0xD7}},
		{x86.R8, []byte{0x41, 0xFF, 0xD0}},
		{x86.R9, []byte{0x41, 0xFF, 0xD1}},
		{x86.R10, []byte{0x41, 0xFF, 0xD2}},
		{x86.R11, []byte{0x41, 0xFF, 0xD3}},
		{x86.R12, []byte{0x41, 0xFF, 0xD4}},
		{x86.R13, []byte{0x41, 0xFF, 0xD5}},
		{x86.R14, []byte{0x41, 0xFF, 0xD6}},
		{x86.R15, []byte{0x41, 0xFF, 0xD7}},
	}

	for _, pattern := range usagePatterns {
		t.Logf("call %s", pattern.Register)
		code := x86.CallRegister(nil, pattern.Register)
		assert.DeepEqual(t, code, pattern.Code)
	}
}
